import React, { FC, Fragment, useEffect, useMemo, useState } from 'react';
import { range } from 'ramda';
import {
	add,
	endOfMonth,
	endOfWeek,
	getMonth,
	getWeekOfMonth,
	getWeeksInMonth,
	getYear,
	startOfMonth,
	startOfWeek,
	sub,
} from 'date-fns';

import AngleLeftIcon from '@prokyon/components/Icons/AngleLeft';
import AngleRightIcon from '@prokyon/components/Icons/AngleRight';
import TodayIcon from '@prokyon/components/Icons/Today';
import { isNilOrEmpty } from '@prokyon/utils';

import { Title, TitleSizeEnum } from '../../Title';
import { WEEK_DAYS_SHORT } from '../constants';
import { DayViewProps } from './DayView';
import { WeekView } from './WeekView';

export type CalendarProps = {
	TimeColumn?: FC;
	View?: FC<DayViewProps>;
	borderSpace?: boolean;
	hover?: boolean;
	onClick?: (date: Date) => void;
	onRangeChange?: (dateFrom: Date, dateEnd: Date) => void;
	timeColumnWidth?: string | number;
	title?: string;
	type?: CalendarViewEnum;
	value?: Date;
};

export enum CalendarViewEnum {
	month = 'month',
	week = 'week',
}

const todayDate: Date = new Date();

export function Calendar({
	value,
	onClick,
	onRangeChange,
	type = CalendarViewEnum.month,
	title,
	View,
	TimeColumn,
	timeColumnWidth,
	hover,
	borderSpace,
}: CalendarProps): ReturnType<FC> {
	const [currentDate, setCurrentDate] = useState<Date>(value ?? todayDate);

	const [start, end] = useMemo((): number[] => {
		if (type === CalendarViewEnum.month) {
			return [0, getWeeksInMonth(currentDate)];
		} else if (type === CalendarViewEnum.week) {
			return [getWeekOfMonth(currentDate), getWeekOfMonth(currentDate) + 1];
		}
		return [0, 0];
	}, [currentDate]);

	const handleChangeRange = () => {
		if (type === CalendarViewEnum.month) {
			onRangeChange?.(startOfMonth(currentDate), endOfMonth(currentDate));
		} else if (type === CalendarViewEnum.week) {
			onRangeChange?.(
				startOfWeek(currentDate, { weekStartsOn: 1 }),
				endOfWeek(currentDate, { weekStartsOn: 1 })
			);
		}
	};

	const handleClickPrev = () => {
		if (type === CalendarViewEnum.month) {
			setCurrentDate(sub(currentDate, { months: 1 }));
		} else if (type === CalendarViewEnum.week) {
			setCurrentDate(sub(currentDate, { weeks: 1 }));
		}
	};

	const handleClickNext = () => {
		if (type === CalendarViewEnum.month) {
			setCurrentDate(add(currentDate, { months: 1 }));
		} else if (type === CalendarViewEnum.week) {
			setCurrentDate(add(currentDate, { weeks: 1 }));
		}
	};

	const handleClickSetToday = () => {
		setCurrentDate(todayDate);
	};

	const handleClick = (date: Date) => {
		onClick?.(date);
	};

	useEffect(() => {
		handleChangeRange();
	}, [currentDate]);

	return (
		<>
			{!isNilOrEmpty(title) && <Title size={TitleSizeEnum.md}>{title}</Title>}
			<div className="text-right pb-2.5 select-none">
				<div className="inline-block px-5">
					<b>{('0' + (getMonth(currentDate) + 1)).slice(-2)}</b> {getYear(currentDate)}
				</div>
				<div
					className="inline-block cursor-pointer pt-1.5 pr-2"
					onClick={handleClickPrev}
				>
					<AngleLeftIcon />
				</div>
				<div
					className="inline-block cursor-pointer text-md"
					onClick={handleClickSetToday}
				>
					<TodayIcon />
				</div>
				<div
					className="inline-block cursor-pointer pt-1.5 pl-2"
					onClick={handleClickNext}
				>
					<AngleRightIcon />
				</div>
			</div>
			{type === CalendarViewEnum.week && TimeColumn ? <TimeColumn /> : <Fragment />}
			<div style={{ ...(timeColumnWidth ? { marginLeft: timeColumnWidth } : {}) }}>
				<div className="table w-full table-fixed border-collapse">
					<div className="table-row-group">
						<div className="table-row">
							{WEEK_DAYS_SHORT.map((day) => (
								<div
									className="table-cell uppercase pb-1.5 text-center font-bold"
									key={day}
								>
									{day}
								</div>
							))}
						</div>
						{range(start, end).map((i: number) => (
							<div className="table-row" key={i}>
								<WeekView
									row={i}
									type={type}
									hover={hover}
									View={View}
									onClick={handleClick}
									todayDate={todayDate}
									currentDate={currentDate}
									borderSpace={borderSpace}
									value={value}
								/>
							</div>
						))}
					</div>
				</div>
			</div>
		</>
	);
}
