import React, { FC, ReactElement, useMemo } from 'react';
import { range } from 'ramda';
import { getDate, getDay, getDaysInMonth, setDate, startOfMonth } from 'date-fns';

import { CalendarViewEnum } from './Calendar';
import { DayView, DayViewProps } from './DayView';

export type WeekViewProps = {
	View?: FC<DayViewProps>;
	borderSpace?: boolean;
	currentDate: Date;
	hover?: boolean;
	onClick?: (date: Date | undefined) => void;
	row: number;
	todayDate: Date;
	type?: CalendarViewEnum;
	value?: Date;
};

export function WeekView(props: WeekViewProps): ReactElement {
	const { type, View, hover, onClick, currentDate, todayDate, row, borderSpace, value } =
		props;

	const firstDayOfMonth = startOfMonth(currentDate).getDay() - 1;
	const daysInMonth = getDaysInMonth(currentDate);

	const getStartDayOfWeek = (rowNum = 0): number => {
		if (type === CalendarViewEnum.month) {
			return 7 - firstDayOfMonth + (rowNum - 1) * 7;
		} else if (type === CalendarViewEnum.week) {
			return getDate(currentDate) - getDay(currentDate);
		}
		return 0;
	};

	const handleChangeDate =
		(dayNum = 0) =>
		(date: Date): void => {
			onClick?.(setDate(date, dayNum));
		};

	const renderDays = useMemo(() => {
		let day = 0;
		if (row !== 0) {
			day = getStartDayOfWeek(row);
		}
		return range(0, 7).map((i) => {
			if (
				(firstDayOfMonth <= i && row === 0) ||
				(daysInMonth > day && row !== 0) ||
				type === CalendarViewEnum.week
			) {
				day++;
				const date = setDate(currentDate, day);
				const props = {
					date,
					isSelected: date.getTime() === value?.getTime(),
					isToday: date.getTime() === todayDate.getTime(),
					hover: hover,
					borderSpace: borderSpace,
					onClick: handleChangeDate(day),
					dayInWeek: i,
					children: day,
					weekInMonth: row,
				};
				return View ? (
					<View key={date.toISOString()} {...props} />
				) : (
					<DayView key={date.toISOString()} {...props} />
				);
			} else {
				return <div key={i} className="table-cell" />;
			}
		});
	}, [
		currentDate,
		todayDate,
		View,
		type,
		borderSpace,
		row,
		value,
		hover,
		firstDayOfMonth,
		daysInMonth,
	]);

	return <>{renderDays}</>;
}
