import React, { useCallback, useEffect, useState } from 'react';
import clsx from 'clsx';

import { StatusEnum } from '@prokyon/types/common';
import { InputElementType } from '@prokyon/types/form';

import CheckSquareIcon from './Icons/CheckSquare';
import SquareIcon from './Icons/Square';

export type CheckboxProps<V = boolean> = Omit<
	{ checked?: boolean; description?: string } & InputElementType<V>,
	'value'
>;

export const Checkbox: React.FC<CheckboxProps> = (props) => {
	const { label, description, name, status = StatusEnum.none, disabled } = props;

	const [checked, setChecked] = useState(false);

	useEffect(() => {
		setChecked(props.checked || false);
	}, [props.checked]);

	const handleClick = useCallback(
		(e: React.MouseEvent<HTMLElement>): void => {
			if (props.onChange) {
				props.onChange(!checked, e);
			}
			setChecked(!checked);
		},
		[checked]
	);

	const handleBlur = useCallback(
		(e: React.FocusEvent<HTMLInputElement>): void => {
			if (props.onBlur) {
				props.onBlur(checked, e);
			}
		},
		[checked]
	);

	const boxClassName = clsx('text-xl field-checkbox', {
		'field-checkbox--info': status === StatusEnum.info,
		'field-checkbox--warning': status === StatusEnum.warning,
		'field-checkbox--error': status === StatusEnum.error,
	});

	return (
		<div className="field-container checkbox-container">
			<input
				id={name}
				name={name}
				className="hidden field-input"
				type="checkbox"
				onBlur={handleBlur}
				disabled={disabled}
			/>
			{label && (
				<label className="field-label mr-2 " htmlFor={name} onClick={handleClick}>
					{label}
				</label>
			)}
			<div className="flex justify-between items-center mr-1 field-checkbox" onClick={handleClick}>
				{checked ? <CheckSquareIcon className={boxClassName} /> : <SquareIcon className={boxClassName} />}
			</div>
			<div className="cursor-pointer" onClick={handleClick}>
				{description}
			</div>
		</div>
	);
};
