import React, { createContext, FC, ReactNode, useMemo, useState } from 'react';
import { omit } from 'ramda';

export type ModalId = string;

export type Modal<P> = {
	Component: FC<P>;
	id: ModalId;
	props?: P;
	style?: {
		overrideMaxWidth?: boolean;
		width?: number | string;
	};
	title?: string | ReactNode;
};

export type AdditionalModalProps<P> = {
	props?: Modal<P>['props'];
	style?: Modal<P>['style'];
};

export type ModalContextType = {
	close: (modalId: ModalId) => void;
	open: <P>(modalId: ModalId, modal?: AdditionalModalProps<P>) => void;
	opened: ModalId[];
	register: <P>(modal: Modal<P>) => void;
	registered: Record<ModalId, Modal<any>>;
	unregister: (modalId: ModalId) => void;
};

export const ModalContext = createContext<ModalContextType>({
	registered: {},
	opened: [],
	open: () => {},
	close: () => {},
	register: () => {},
	unregister: () => {},
});

export const ModalWrapper: FC<{ children: ReactNode }> = ({ children }) => {
	const [modals, setModals] = useState<Record<ModalId, Modal<any>>>({});
	const [opened, setOpened] = useState<ModalId[]>([]);

	const register = function <P>(modal: Modal<P>) {
		setModals({ ...modals, [modal.id]: modal });
	};

	const unregister = (modalId: ModalId) => {
		setModals(omit([modalId], modals));
	};

	const open = function <P>(modalId: ModalId, modal?: AdditionalModalProps<P>) {
		if (modal) {
			setModals({
				...modals,
				[modalId]: {
					...modals[modalId],
					...modal,
				},
			});
		}
		setOpened([...opened, modalId]);
	};

	const close = (modalId: ModalId) => {
		setOpened(opened.filter((id) => id !== modalId));
	};

	const value = useMemo(
		() => ({
			registered: modals,
			opened,
			unregister,
			register,
			open,
			close,
		}),
		[modals, opened]
	);

	return <ModalContext.Provider value={value}>{children}</ModalContext.Provider>;
};
