import React, { useCallback, useEffect, useState } from 'react';
import { find, propEq, propOr } from 'ramda';

import { ModalId } from '@prokyon/components/Modal/context';
import { useModal } from '@prokyon/components/Modal/hooks';
import { InputElementType } from '@prokyon/types/form';
import { Option } from '@prokyon/types/options';

import { TextField } from '../../TextField';
import { SelectboxModal } from './SelectboxModal';

export type SelectBoxProps<V = Option> = Omit<
	{
		autoComplete?: boolean;
		options: Option[];
	} & InputElementType<V>,
	'onFocus' | 'onClick'
>;

export const Selectbox: React.FC<SelectBoxProps> = (props) => {
	const { value, options, onBlur, onChange, name, label, autoComplete, ...others } =
		props;

	const { openModal, closeModal, registerModal } = useModal();

	const [selectedOption, setSelectedOption] = useState<Option | null>(null);

	useEffect(() => {
		if (value && options) {
			const newValue = find<any>(propEq('code', value), options);
			setSelectedOption(newValue);
		}
	}, [value, options]);

	const handleBlur = (_: string, e: React.FocusEvent<HTMLInputElement>): void => {
		onBlur?.(selectedOption ?? undefined, e);
	};

	const handleOptionClick = (option: Option): void => {
		setSelectedOption(option);
		onChange?.(option, null);
	};

	const handleOptionRemove = (): void => {
		setSelectedOption(null);
		onChange?.(undefined, null);
	};

	const handleCloseModal = (id: ModalId) => {
		closeModal(id);
	};

	const modalComponent = useCallback(
		(props) => (
			<SelectboxModal
				autoComplete={autoComplete}
				name={name}
				options={options}
				selectedOption={selectedOption}
				closeModal={handleCloseModal}
				onChange={handleOptionClick}
				onRemove={handleOptionRemove}
				{...props}
			/>
		),
		[autoComplete, name, options, selectedOption]
	);

	const handleOpenModal = (): void => {
		registerModal({
			id: name,
			title: label,
			style: { width: 300 },
			Component: modalComponent,
		});
		openModal(name);
	};

	return (
		<TextField
			{...others}
			name={name}
			label={label}
			onClick={handleOpenModal}
			onFocus={handleOpenModal}
			onBlur={handleBlur}
			value={propOr('', 'name', selectedOption)}
			readonly
		/>
	);
};
