import React, {
	ReactChild,
	ReactNode,
	useCallback,
	useEffect,
	useMemo,
	useState,
} from 'react';
import { isNil } from 'ramda';
import clsx from 'clsx';
import { BrowserHistory } from 'history';

import MenuIcon from '@prokyon/components/Icons/Menu';
import RightToBracketIcon from '@prokyon/components/Icons/RightToBracket';
import Modals from '@prokyon/components/Modal/components/Modals';
import Toasters from '@prokyon/components/Toaster/components/Toasters';
import { SIZES, useMediaQuery } from '@prokyon/hooks/useMediaQuery';
import { isNilOrEmpty } from '@prokyon/utils';

import { MenuItem } from '../types';
import { Item } from './Item';

type IProps = {
	children: ReactChild;
	components?: {
		footer?: React.ReactElement;
	};
	history: BrowserHistory;
	items?: {
		logo?: ReactNode;
		side?: MenuItem[];
		top?: MenuItem[];
		user?: MenuItem[];
	};
};

const Skeleton: React.FC<IProps> = (props) => {
	const { children, components = {}, items, history } = props;

	const { footer } = components;

	const enabledSide = !isNilOrEmpty(items?.side);
	const enabledTop = !isNilOrEmpty(items?.top);
	const enabledUser = !isNilOrEmpty(items?.user);

	const [topID, setTopOpen] = useState<string | null>(null);
	const [sideID, setSideOpen] = useState<string | null>(null);

	const { isMaxMatch: isMobile } = useMediaQuery({ size: SIZES.md });
	const { isMaxMatch: isTablet } = useMediaQuery({ size: SIZES.lg });

	const [sideCollapsed, setSideCollapsed] = useState(isMobile || isTablet);
	const [topCollapsed, setTopCollapsed] = useState(isMobile);

	useEffect(() => {
		setSideCollapsed(isMobile || isTablet);
		setTopCollapsed(isMobile);
	}, [isMobile, isTablet]);

	const toggleSide = () => {
		setSideCollapsed(!sideCollapsed);
	};

	const toggleTop = () => {
		setTopCollapsed(!topCollapsed);
	};

	const openLink = useCallback(
		({ href, external, onClick }: MenuItem) => {
			if (!isNil(onClick)) {
				onClick();
			} else {
				if (external) {
					window.open(href, '_blank');
				} else if (href) {
					history.push(href);
				}
			}
		},
		[history]
	);

	const sideItems = useMemo(() => {
		return items?.side?.map((item, i) => (
			<Item
				key={item.href ?? i}
				isChild={false}
				onClick={openLink}
				item={item}
				id={sideID}
				setOpen={setSideOpen}
				collapsed={sideCollapsed}
			/>
		));
	}, [items?.side, sideID, sideCollapsed]);

	const topItems = useMemo(() => {
		return items?.top?.map((item, i) => (
			<Item
				key={item.href ?? i}
				id={topID}
				setOpen={setTopOpen}
				isChild={false}
				onClick={openLink}
				item={item}
			/>
		));
	}, [items?.top, topID]);

	const userItems = useMemo(() => {
		return items?.user?.map((item, i) => (
			<Item
				key={item.href ?? i}
				id={topID}
				setOpen={setTopOpen}
				isChild={false}
				onClick={openLink}
				className="menu-item-user"
				item={item}
			/>
		));
	}, [items?.user, topID]);

	return (
		<div className="app">
			{enabledTop && (
				<div className="page-top">
					{items?.logo && (
						<div className="page-top-logo">
							<div>{items?.logo}</div>
						</div>
					)}
					<div className="page-top-content">
						{isMobile && <MenuIcon className="top-toggler" onClick={toggleTop} />}
						{!topCollapsed && (
							<>
								{topItems}
								{enabledUser && <div className="menu-delimiter" />}
								{userItems}
							</>
						)}
					</div>
				</div>
			)}
			<div className={clsx('page-main', { flex: enabledSide })}>
				{enabledSide && (
					<div
						className={clsx('side', {
							'side--collapsed': sideCollapsed,
							'side--expand': !sideCollapsed,
						})}
					>
						<RightToBracketIcon
							className={clsx('side-toggler', { 'rotate-180': !sideCollapsed })}
							onClick={toggleSide}
						/>
						{sideItems}
					</div>
				)}
				<div className="content">
					{children}
					<div className="mt-4">{footer}</div>
				</div>
			</div>
			<Modals />
			<Toasters />
		</div>
	);
};

Skeleton.displayName = 'Skeleton';

export default Skeleton;
