import React, {
	ChangeEvent,
	FocusEvent,
	MouseEvent,
	useCallback,
	useEffect,
	useState,
} from 'react';
import clsx from 'clsx';

import { StatusEnum } from '@prokyon/types/common';
import { InputElementType } from '@prokyon/types/form';
import { masker } from '@prokyon/utils';

export type TextFieldProps<V = string | number> = {
	autoFocus?: boolean;
	mask?: string;
	multiline?: boolean;
	readonly?: boolean;
	type?: 'text' | 'password';
} & InputElementType<V>;

export const TextField: React.FC<TextFieldProps> = (props) => {
	const {
		type = 'text',
		onClick,
		onFocus,
		onBlur,
		onChange,
		disabled,
		name,
		label,
		readonly,
		status,
		autoFocus,
		mask,
		multiline,
	} = props;
	const [value, setValue] = useState<typeof props.value>(undefined);

	const applyMask = (val: typeof props.value) => {
		if (mask && val) {
			return masker.toPattern(val.toString(), mask);
		}
		return val;
	};

	useEffect((): void => {
		if (props.value !== undefined) {
			setValue(applyMask(props.value));
		}
	}, [props.value]);

	const handleClick = useCallback(
		(e: MouseEvent<HTMLInputElement | HTMLTextAreaElement>): void => {
			if (onClick) {
				onClick(e.currentTarget.value, e);
			}
		},
		[onClick]
	);

	const handleFocus = useCallback(
		(e: FocusEvent<HTMLInputElement | HTMLTextAreaElement>): void => {
			if (onFocus) {
				onFocus(e.currentTarget.value, e);
			}
		},
		[onFocus]
	);

	const handleBlur = useCallback(
		(e: FocusEvent<HTMLInputElement | HTMLTextAreaElement>): void => {
			if (onBlur) {
				onBlur(e.currentTarget.value, e);
			}
		},
		[onBlur]
	);

	const handleChange = (e: ChangeEvent<HTMLInputElement | HTMLTextAreaElement>): void => {
		const val = e.currentTarget.value;
		if (!disabled) {
			setValue(applyMask(val));
			if (onChange) {
				onChange(applyMask(val), e);
			}
		}
	};

	const calculateHeight = (e: HTMLTextAreaElement) => {
		const count = (e.value.match(/\n/g) ?? []).length + 1;
		const height = parseInt(window.getComputedStyle(e).lineHeight);
		if (count === 1) {
			e.style.height = '';
		} else {
			e.style.height = `${height * count + height / 2}px`;
		}
	};

	const passProps = {
		id: name,
		className: clsx('field-input masked', {
			'field-input--nolabel': !label,
			'field-input--multiline': multiline,
		}),
		disabled: disabled,
		readOnly: readonly,
		value: value ?? '',
		onClick: handleClick,
		onFocus: handleFocus,
		onBlur: handleBlur,
		onChange: handleChange,
		autoFocus: autoFocus,
		name: name,
	};

	return (
		<div
			className={clsx('field-container', {
				'field-container--multiline': multiline,
				'field-container--info': status === StatusEnum.info,
				'field-container--success': status === StatusEnum.success,
				'field-container--warning': status === StatusEnum.warning,
				'field-container--error': status === StatusEnum.error,
			})}
		>
			{label && (
				<label className={clsx('field-label', { 'field-label--multiline': multiline })}>
					{label}
				</label>
			)}
			{multiline ? (
				<textarea {...passProps} ref={(ref) => ref && calculateHeight(ref)} />
			) : (
				<input {...passProps} autoComplete="off" type={type} />
			)}
		</div>
	);
};
