import React, { FC, FocusEvent, useCallback, useEffect, useState } from 'react';
import { format } from 'date-fns';

import { useModal } from '@prokyon/components/Modal/hooks';
import { InputElementType } from '@prokyon/types/form';
import { Option } from '@prokyon/types/options';

import { TextField } from '../../TextField';
import { TimePickerModal } from './TimePickerModal';

export type TimePickerProps<V = string> = Omit<
	{
		hour?: {
			max?: number;
			min?: number;
			step?: number;
		};
		minute?: {
			max?: number;
			min?: number;
			step: number;
		};
		scrollToCode?: Option['code'];
	} & InputElementType<V>,
	'onFocus' | 'onClick'
>;

export const TimePicker: FC<TimePickerProps<Date>> = (props) => {
	const { onBlur, onChange, label, name, hour, minute, scrollToCode, ...others } = props;

	const { openModal, registerModal } = useModal();

	const [value, setValue] = useState<Date>();

	useEffect(() => {
		setValue(props.value);
	}, [props.value]);

	const handleBlur = (value: string, e: FocusEvent): void => {
		onBlur?.(new Date(value), e);
	};

	const handleTimeClick = (value: Date): void => {
		setValue(value);
		onChange?.(value, null);
	};

	const handleOpenModal = (): void => {
		registerModal({
			id: name,
			title: label,
			style: { width: 250 },
			Component: modalComponent,
		});
		openModal(name);
	};

	const modalComponent = useCallback(
		(props) => (
			<TimePickerModal
				value={value}
				onChange={handleTimeClick}
				scrollToCode={scrollToCode}
				modalName={name}
				minute={{ step: 1, min: 0, max: 59, ...(minute ?? {}) }}
				hour={{ step: 1, min: 0, max: 23, ...(hour ?? {}) }}
				{...props}
			/>
		),
		[value, scrollToCode, name, minute, hour]
	);

	return (
		<TextField
			{...others}
			name={name}
			label={label}
			onClick={handleOpenModal}
			onFocus={handleOpenModal}
			onBlur={handleBlur}
			value={value ? format(value, 'HH:mm') : ''}
			readonly
		/>
	);
};
