import React, { useEffect, useState } from 'react';
import { format, setHours, setMinutes, setSeconds } from 'date-fns';

import Button from '@prokyon/components/Button';
import { Grid, GridSizeEnum } from '@prokyon/components/Grid';
import { GridCol } from '@prokyon/components/GridCol';
import TrashIcon from '@prokyon/components/Icons/Trash';
import { useModal } from '@prokyon/components/Modal/hooks';
import { t } from '@prokyon/localization/message';
import { StatusEnum } from '@prokyon/types/common';
import { Option } from '@prokyon/types/options';

import TimeList from './TimeList';

export type TimePickerModalProps = {
	hour: {
		max: number;
		min: number;
		step: number;
	};
	minute: {
		max: number;
		min: number;
		step: number;
	};
	modalName: string;
	onChange: (value: Date | undefined) => void;
	scrollToCode?: Option['code'];
	value?: Date;
};

const { base } = GridSizeEnum;

export const TimePickerModal: React.FC<TimePickerModalProps> = (props) => {
	const { onChange, modalName, scrollToCode } = props;

	const { closeModal } = useModal();

	const [hour, setHour] = useState<number | null>();
	const [minute, setMinute] = useState<number | null>();

	useEffect(() => {
		if (props.value) {
			const value = format(props.value, 'HH:mm');
			const hour = value.split(':')[0];
			const minute = value.split(':')[1];
			setHour(parseInt(hour));
			setMinute(parseInt(minute));
		}
	}, [props.value]);

	const handleSetHour = (value: number) => {
		setHour(value);
	};

	const handleSetMinute = (value: number) => {
		setMinute(value);
	};

	const handleChangeValue = () => {
		let value = props.value ?? new Date();
		value = setHours(value, hour ?? 0);
		value = setMinutes(value, minute ?? 0);
		value = setSeconds(value, 0);
		onChange(value);
		closeModal(modalName);
	};

	const handleClearValue = () => {
		setHour(null);
		setMinute(null);
		onChange(undefined);
		closeModal(modalName);
	};

	const selectedHour = hour ? hour : undefined;
	const selectedMinute = minute ? minute : undefined;

	return (
		<>
			<Grid cols={{ [base]: 2 }}>
				<GridCol>
					<TimeList
						name="hour"
						title={t('prokyon.components.timePicker.hours')}
						onChange={handleSetHour}
						{...props.hour}
						selectedCode={selectedHour}
						scrollToCode={scrollToCode}
					/>
				</GridCol>
				<GridCol>
					<TimeList
						name="minute"
						title={t('prokyon.components.timePicker.minutes')}
						onChange={handleSetMinute}
						{...props.minute}
						selectedCode={selectedMinute}
						scrollToCode={scrollToCode}
					/>
				</GridCol>
			</Grid>
			<Grid cols={{ [base]: 2 }} className="mt-4">
				<GridCol>
					<Button label="Uložit" status={StatusEnum.success} onClick={handleChangeValue} />
				</GridCol>
				<GridCol>
					<Button label={<TrashIcon fontSize={20} />} status={StatusEnum.warning} onClick={handleClearValue} />
				</GridCol>
			</Grid>
		</>
	);
};
